;**********************************************************************
;                                                                     	*
;    Filename:                                                        	*	   
;    ELMA.asm                                          					*
;                                                                     	*
;    Date:                                                              *   
;    30/09/09                                          					*
;                                                                     	*
;    File Version:                                                      * 
;    1.0                                               					*
;                                                                     	*
;    Author:                                                          	*     
;    Justin Grimm                                      					*
;                                                                     	*
;    Multiplier subroutine Adapted from 								*
;    http://www.convict.lu/Jeunes/Math/Fast_operations.htm				*
;    Claude Baumann 2002; updated 2006 thanks to L. Armstrong           * 
;                                                                     	*
;**********************************************************************
;         							      								*
;**********************************************************************
;                                                                     	*
;    Notes:     Port assignments               tris                   	*
;                                                                     	*
;	RA0			AN0		Position				1                     	*
;	RA1     	AN1   	PWM09           		0                     	*
;   RA2         AN2     PWM10	        		0                     	*
;   RA3         AN3     PWM11           		0                     	*
;   RA4					PWM12					0		      			*	
;	RA5			AN4		PWM13     				0		      			*
;	RB0					PWM1					0		      			*	
;	RB1					PWM2					0		      			*
;	RB2					PWM3					0                     	*
;	RB3					PWM4 					0                     	*
;	RB4					PWM5					0                     	*
;	RB5					PWM6					0                     	* 
;	RB6					PWM7					0	              		*
;	RB7					PWM8					0                    	*
;	RC0											0                     	*
;	RC1											0                     	*
;	RC2			  								0                     	*
;	RC3			  								0                     	*
;	RC4											0                     	*
;	RC5											0                     	*
;	RC6					232/tx					0                     	* 
;	RC7					232/rx					1                     	* 
;	RD0											0                     	*
;	RD1											0                     	*	 
;	RD2											0                     	*
;	RD3											0                     	*
;	RD4											0                     	*
;	RD5											0		      			*
;	RD6					     					0                     	*
;	RD7						 					0                     	*
;	RE0					AN5						0                     	*
;	RE1					AN6						0                     	* 
;	RE2					AN7						0	              		*
;                                                                     	*
;**********************************************************************
;         							      								*
; rs232 receive commands												*
;         							      								*
;         							      								*
; 0xf7	position requested												*
; 0xf8	sp requested													*
;         							      								*
;**********************************************************************



;***** VARIABLE DEFINITIONS

w			equ	0
f			equ	1
w_temp      EQU     0x20        		; variable used for context saving 
status_temp EQU     0x21        		; variable used for context saving
pclath_temp	EQU		0x22        		; variable used for context saving
pclath		equ	0x0a
status		equ	0x03
z			equ	2
c			equ	0
dc			equ	1
rp0			equ	5
tmr0		equ	0x01
pcl			equ	0x02
fsr			equ	0x04
porta		equ	0x05
pot1		equ	0
PWM09		equ	1
PWM10		equ	2
PWM11		equ	3
PWM12		equ	4
PWM13		equ	5
portb		equ	0x06
PWM01		equ	0
PWM02		equ	1
PWM03		equ	2
PWM04		equ	3
PWM05		equ	4
PWM06		equ	5
PWM07		equ	6
PWM08		equ	7
portc		equ	0x07
portd		equ	0x08
porte		equ	0x09
intcon		equ	0x0b 
rbif		equ	0
intf		equ	1
t0if		equ	2
rbie		equ	3
inte		equ	4
t0ie		equ	5
peie		equ	6
gie			equ	7
pir1		equ	0x0c
tmr1if		equ	0
tmr2if		equ	1
ccp1if		equ	2
sspif		equ	3
txif		equ	4
rcif		equ	5
adif		equ	6
pspif		equ	7
pir2		equ	0x0d
tmr1l		equ	0x0e
tmr1h		equ	0x0f
t1con		equ	0x10
tmr2		equ	0x11
t2con		equ	0x12
sspbuf		equ	0x13
sspcon		equ	0x14
ccpr1l		equ	0x15
ccpr1h		equ	0x16
ccp1con		equ	0x17
rcsta		equ	0x18
txreg		equ	0x19
rcreg		equ	0x1a
ccpr2l		equ	0x1b
ccpr2h		equ	0x1c
ccp2con		equ	0x1d
adresh		equ	0x1e
adcon0		equ	0x1f
option_reg	equ	0x81
trisa		equ	0x85
trisb		equ	0x86
trisc		equ	0x87
trisd		equ	0x88
trise		equ	0x89
pie1		equ	0x8c
pie2		equ	0x8d
pcon		equ	0x8e
pr2			equ	0x92
sspadd		equ	0x93
sspstat		equ	0x94
txsta		equ	0x98
txen		equ	5
spbrg		equ	0x99
adresl		equ	0x9e
adcon1		equ	0x9f	
txdata 		equ	0x23					; data to be transmitted over sci
rcsave 		equ	0x24					; register to save received byte drom sci
pos_sp 		equ	0x25					; the position setpoint in mm
position	equ	0x26					; actual position in mm
pos_error	equ	0x27					; absolute position error
flags 		equ	0x28					; general purpose flags register
move_up_req	equ	0						; means position > pos_sp 
pos_add		equ	1						; added to position for hysteresis adjustment last scan
tempx16		equ	0x29					; multiplier 1 for 16 bit multiplier sub
tempy16		equ	0x2a					; multiplier 2 for 16 bit multiplier sub
wait4		equ	0x2b					; Temporary register
temp1		equ	0x2c					; Temporary register
result16_L	equ	0x2d					; Result LSB for 16 bit multiplier sub
result16_H	equ	0x2e					; Result MSB for 16 bit multiplier sub
tmr0_cnt	equ	0x2f					; count to determine when the ISR has been called 32 times. This is the 1kHz PWM period.
PWM01_SP	equ	0x30					; PWM setpoints for each coil. 0=fully reverse, 16=off, 31=fully forward.
PWM02_SP	equ	0x31
PWM03_SP	equ	0x32
PWM04_SP	equ	0x33
PWM05_SP	equ	0x34
PWM06_SP	equ	0x35
PWM07_SP	equ	0x36
PWM08_SP	equ	0x37
PWM09_SP	equ	0x38
PWM10_SP	equ	0x39
PWM11_SP	equ	0x3a
PWM12_SP	equ	0x3b
PWM13_SP	equ	0x3c
drive_byte1	equ	0x3d					; Drive data from table read
drive_byte2	equ	0x3e
drive_byte3	equ	0x3f
drive_byte4	equ	0x4a
tempx16_H	equ	0x4b					; temporary registers from multiplier routine
tempy16_H	equ	0x4c
adresl_reg	equ	0x4d					; holding register for A/D converter low byte
pos_last	equ	0x4e					; last scan position
pos_temp	equ	0x4f					; temporary position location




;**********************************************************************
; PROGRAM START

	ORG     0x000         	    		; processor reset vector
	clrf    pclath         	   			; ensure page bits are cleared
  	goto    boot           	  			; go to beginning of program



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;ISR
;;;;This interrupt vector is called every 31.25us (1kHz/32) and services the PWM outputs.
;;;;PWM is 0-100% duty cycle with 50% being off, 0% being fully reverse and 100% being fully forward.
;;;;Resolution is a total of 32 steps, 0-31 being 0-100% duty cycle. Therefore 0-15 is 100% to 0% reverse
;;;;and 16-31 is 0% to 100% forward.

	ORG     0x004            			; interrupt vector location

	MOVWF 	w_temp 						;Copy W to TEMP register
	SWAPF 	status,w 					;Swap status to be saved into W
	CLRF 	status 						;bank 0, regardless of current bank, Clears IRP,RP1,RP0
	MOVWF 	status_temp 				;Save status to bank zero STATUS_TEMP register
	MOVF 	pclath,w 					;Only required if using pages 1, 2 and/or 3
	MOVWF 	pclath_temp 				;Save PCLATH into W
	CLRF 	pclath 						;Page zero, regardless of current page

isr
	movlw	0x5d						; 256-156-2-5 = 0x5d. Timer transitions FF to 00 after 156 instruction cycles = 31.2us.
	movwf	tmr0
	incfsz	tmr0_cnt,f					; increment the PWM period counter, skip next instruction if transition to zero. (2 Tcy)
	goto	pwm_period_not_fin			; 1ms period not finished (2 Tcy)
	goto 	pwm_period_fin				; 1ms period finished (2 Tcy)

pwm_period_not_fin
	movlw	0xe1						; tmr0_cnt - 0xe1 -> temp1
	subwf	tmr0_cnt,w					; result is 0-31.
	movwf	temp1
	
	movf	temp1,w
	subwf	PWM01_SP,w					; PWM01_SP - temp1 -> w. If c = 0 then temp1 > PWM01_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM01

	movf	temp1,w
	subwf	PWM02_SP,w					; PWM02_SP - temp1 -> w. If c = 0 then temp1 > PWM02_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM02

	movf	temp1,w
	subwf	PWM03_SP,w					; PWM03_SP - temp1 -> w. If c = 0 then temp1 > PWM03_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM03

	movf	temp1,w
	subwf	PWM04_SP,w					; PWM04_SP - temp1 -> w. If c = 0 then temp1 > PWM04_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM04

	movf	temp1,w
	subwf	PWM05_SP,w					; PWM05_SP - temp1 -> w. If c = 0 then temp1 > PWM05_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM05

	movf	temp1,w
	subwf	PWM06_SP,w					; PWM06_SP - temp1 -> w. If c = 0 then temp1 > PWM06_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM06

	movf	temp1,w
	subwf	PWM07_SP,w					; PWM07_SP - temp1 -> w. If c = 0 then temp1 > PWM07_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM07

	movf	temp1,w
	subwf	PWM08_SP,w					; PWM08_SP - temp1 -> w. If c = 0 then temp1 > PWM08_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	portb,PWM08

	movf	temp1,w
	subwf	PWM09_SP,w					; PWM09_SP - temp1 -> w. If c = 0 then temp1 > PWM09_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	porta,PWM09

	movf	temp1,w
	subwf	PWM10_SP,w					; PWM10_SP - temp1 -> w. If c = 0 then temp1 > PWM10_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	porta,PWM10

	movf	temp1,w
	subwf	PWM11_SP,w					; PWM11_SP - temp1 -> w. If c = 0 then temp1 > PWM11_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	porta,PWM11

	movf	temp1,w
	subwf	PWM12_SP,w					; PWM12_SP - temp1 -> w. If c = 0 then temp1 > PWM12_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	porta,PWM12

	movf	temp1,w
	subwf	PWM13_SP,w					; PWM13_SP - temp1 -> w. If c = 0 then temp1 > PWM13_SP and the PWM output should be 0.
	btfss	status,c					; test carry flag, if zero then clear PWM output.
	bcf	porta,PWM13

	goto	end_isr


pwm_period_fin
	movlw	0xe0						; reset counter to 256-32=0xe0
	movwf	tmr0_cnt		 
	movlw	0xff						; set all PWM outputs to 1 on port b
	movwf	portb
	movlw	0x1f						; set PWM9, 10, 11, 12 and 13 to 1 on port a
	iorwf	porta,f


end_isr
	bcf		intcon,t0if					; clear interrupt flag

	MOVF 	pclath_temp, W 				;Restore PCLATH
	MOVWF 	pclath 						;Move W into PCLATH
	SWAPF 	status_temp,w 				;Swap STATUS_TEMP register into W
										;(sets bank to original state)
	MOVWF 	status 						;Move W into STATUS register
	SWAPF 	w_temp,f 					;Swap W_TEMP
	SWAPF 	w_temp,w 					;Swap W_TEMP into W

	retfie                    			; return from interrupt




;;;; ISR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; INITIALISE


init	
	bcf	intcon,gie
	clrf	porta
	clrf	portb
	clrf	portc
	clrf	portd
	clrf	porte
	clrf	position
	clrf	wait4	
	bsf		status,rp0					; bank1
	movlw	0x01						; set port directions- ra0 analogue in
	movwf	trisa
	movlw	0x00						; 00000000
	movwf	trisb			
	movlw	0x80						; 10000000	
	movwf	trisc
	movlw	0x00						; 00000000
	movwf	trisd
	movlw	0x00
	movwf	trise
	movlw	0x89						; RB P/U disabled, prescaler WDT 2:1, int on falling edge, timer 0 uses internal clock
	movwf	option_reg
	bcf		0x83,rp0					; bank0
	movlw	0x59						; initialise position setpoint and last position to 89mm
 	movwf	pos_sp
	movwf	pos_last		
	clrf	result16_L
	clrf	result16_H
	clrf	tmr0_cnt
	clrf	temp1

initad		
	movlw	0x81						; Fosc/32 clk,channel ra0,a/d on
	movwf	adcon0						; 
	bsf		status,rp0					; bank1
	movlw	0x8e						; High bits from result are in lower bits of ADRESH
	movwf	adcon1						; set ra0 to analog

initsci									; txif is set when tx finished, to tx, load txreg check txif
	movlw	0x81						; 2400 baud at 20Mhz
	movwf	spbrg
	movlw	0x20						; clr brgh (low baud rate),async op, enabled
	movwf	txsta						; 
	bcf		0x83,5						; bank0
	movlw	0x80						; enable serial		
	movwf	rcsta						; 
	clrf	txreg						; clr rx tx regs
	clrf	rcreg						;
	movlw	0x90						; enable receive
	movwf	rcsta

initint				
	clrf	intcon						; disable all interrupts
	bsf		intcon,t0ie					; enable interrupt on timer 0
	movlw	0x00						; disable peripheral interrupts
	movwf	pie1

inittmr0
	movlw	0x62						; 256-156-2 = 0x61. Timer transitions FF to 00 after 156 instruction cycles = 31.2us.
	movwf	tmr0
	movlw	0xe0						; load 256-32=0xe0 into ISR counter
	movwf	tmr0_cnt					; 
	movlw	0xff						; set all PWM outputs to 1 on port b
	movwf	portb
	movlw	0x1f						; set PWM9, 10, 11, 12 and 13 to 1 on port a
	iorwf	porta,f

	bsf		intcon,gie					; enable global interrupts (temp disabled)
	
	return

;;;; INITIALISE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; SUBROUTINES

;; Read analogue input. 
;; Inputs: None
;; Outputs: Result in ADRESH and ADRESL.
;; Length: 17 Tcy + wait time for A/D conversion.

get_ad									; result in ADRES reg


	movlw	0x02						; wait 2us sampling time
	movwf	wait4
adloop
	nop
	decfsz	wait4,f
	goto	adloop	
	bsf	adcon0,2						; start ad conversion
waitad
	btfsc	adcon0,2					; wait for a/d conversion	
	goto	waitad
	return

tx_data
	movf	txdata,w					; move transmit data to tx
	movwf	txreg	
wait_tx
	btfsc	pir1,txif					; tx finished?
	return								; yes
	goto	wait_tx						; no
	
	

;; Multiply 2 * 8 bit registers. Adapted from http://www.convict.lu/Jeunes/Math/Fast_operations.htm
;; tempx16 * tempy16 -> result16_H and result16_L

mul16 
        clrf 	result16_L
        clrf 	result16_H
        clrf 	tempx16_H
        clrf 	tempy16_H
mul16_loop
        btfsc 	tempy16,0
        call 	add16
        bcf 	status,c
        rrf 	tempy16_H,f
        rrf 	tempy16,f
        bcf 	status,c
        rlf 	tempx16,f
        rlf 	tempx16_H,f
        movf 	tempy16,f
        btfss 	status,z
        goto 	mul16_loop
        movf 	tempy16_H,f
        btfss 	status,z
        goto 	mul16_loop
        return
add16
        movf 	tempx16,w
        addwf 	result16_L
        btfsc 	status,c
        incf 	result16_H
        movf 	tempx16_H,w
        addwf 	result16_H
        return

;;;; SUBROUTINES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; MAIN PROGRAMME

boot
	clrwdt
	call	init						; initialise device, a/d, sci, int

main

check_sci_rec							; check if a byte received over the sci
	btfss	pir1,rcif					; check for received bit
	goto	get_position				; no
	bcf		pir1,rcif					; yes, clr flag
	movlw	0x80						; disable receive
	movwf	rcsta			
	movf	rcreg,w						; save received byte
	movwf	rcsave

	btfss	rcsave,7					; see if msb 1, valid data requests have the msb as 1.
	goto	_pos_sp_rec					; no, not valid data request, must be position setpoint. Position setpoint is 0-178mm

	movlw	0xf7						; see if current position data wanted
	xorwf	rcsave,w
	btfsc	status,z
	goto	send_position				; yes

	movlw	0xf8						; see if setpoint data wanted
	xorwf	rcsave,w
	btfsc	status,z
	goto	send_sp						; yes

_pos_sp_rec
	movf	rcsave,w					; copy position setpoint to pos_sp register
	movwf	pos_sp
	goto 	check_sci_cont				; continue
	
send_position
	movf	position,w					; send position data to PC.
	movwf	txdata
	call	tx_data
	goto	check_sci_cont

send_sp
	movf	pos_sp,w					; send position data to PC.
	movwf	txdata
	call	tx_data
	goto	check_sci_cont

check_sci_cont
	movlw	0x90						; enable receive
	movwf	rcsta	


get_position
	call	get_ad						; get analogue input raw data

	bsf		status,rp0					; bank1
	movf	adresl,w					; save adresl to bank 0
	bcf		0x83,5						; bank0
	movwf	adresl_reg

	movlw	0xfe						; clamp adresl_reg to a max of 254 if above 254
	subwf	adresl_reg,w				; adresl_reg - 0xfe -> w
	btfss	status,c					; see if adresl_reg < 0xfe
	goto	get_position_cont			; yes, continue
	movlw	0xfe						; no, set adresl_reg to 0xfe
	movwf	adresl_reg

get_position_cont

position_cont2
	movlw	0x00						; see if ad result in bank 0
	xorwf	adresh,w
	btfsc	status,z
	goto	get_pos_bank0				; yes
	movlw	0x01						; see if ad result in bank 1
	xorwf	adresh,w
	btfsc	status,z
	goto	get_pos_bank1				; yes
	movlw	0x02						; see if ad result in bank 2
	xorwf	adresh,w
	btfsc	status,z
	goto	get_pos_bank2				; yes
	movlw	0x03						; see if ad result in bank 3
	xorwf	adresh,w
	btfsc	status,z
	goto	get_pos_bank3				; yes
	goto	get_pos_bank3				; error



get_pos_bank0
	movlw	0x04
	movwf	pclath
	movf	adresl_reg,w
	call	pos_table1					; returns with rotor position in mm in w
	movwf	position					; save actual position to register
;	clrf	pclath
	goto	control
get_pos_bank1
	movlw	0x05
	movwf	pclath
	movf	adresl_reg,w
	call	pos_table2					; returns with rotor position in mm in w
	movwf	position					; save actual position to register
;	clrf	pclath
	goto	control
get_pos_bank2
	movlw	0x06
	movwf	pclath
	movf	adresl_reg,w
	call	pos_table3					; returns with rotor position in mm in w
	movwf	position					; save actual position to register
;	clrf	pclath
	goto	control
get_pos_bank3
	movlw	0x07
	movwf	pclath
	movf	adresl_reg,w
	call	pos_table4					; returns with rotor position in mm in w
	movwf	position					; save actual position to register
;	clrf	pclath
	goto	control

control

	movf	pos_last,w					; see if there was a change in position
	xorwf	position,w
	btfss	status,z	
	goto	_pos_adj_req				; yes
	btfsc	flags,pos_add				; no, see if last scan added or subtracted from position
	goto	_pos_gt_last				; added, retain added value
	goto	_pos_lt_last				; subtracted, retain added value
_pos_adj_req
	movf	position,w					; save raw position
	movwf	pos_temp	
	subwf	pos_last,w					; see if moving up or down, pos_last - position -> w
	btfsc	status,c					; see if carry = 0 (means position > pos_last)
	goto	_pos_lt_last				; position is less than last position
_pos_gt_last
	movlw	0x02						; add 2 to position to account for hysteresis in pot
	addwf	position,f
	bsf		flags,pos_add				; set 'position adjustment added' flag
	goto 	_pos_adj_cont
_pos_lt_last
	movlw	0x02						; subtract 2 from position to account for hysteresis in pot
	subwf	position,f
	bcf		flags,pos_add				; clear 'position adjustment added' flag
_pos_adj_cont
	movf	pos_temp,w					; save raw position to position last
	movwf	pos_last
;;new

	movf	position,w					; find position absolute error
	subwf	pos_sp,w					; pos_sp - position -> w
	movwf	pos_error					; save position error
	bsf		flags,move_up_req			; clear flag ;;reversed
	btfss	status,c					; see if carry = 0 (means position > pos_sp)
	goto	_inverse_error				; yes, find inverse of error
	goto	_control_cont				; no, continue
_inverse_error
	movf	pos_sp,w					; find position absolute error
	subwf	position,w					; position - pos_sp -> w
	movwf	pos_error					; save position error	
	bcf	flags,move_up_req  				; reversed

_control_cont
;	bcf	status,c						; divide pos_error by 2. Code below scales a max error of 178mm to 22
;	rrf	pos_error,f
;	bcf	status,c						; divide pos_error by 2 
;	rrf	pos_error,f
;	bcf	status,c						; divide pos_error by 2
;	rrf	pos_error,f						; **commented out to increase gain**
	movlw	0x0f						; clamp error to a max of 15 if above 15
	subwf	pos_error,w					; pos_error - 0x0f -> w
	btfss	status,c					; see if pos_error < 0x0f
	goto	get_drive_data				; yes, continue
	movlw	0x0f						; no, set pos_error to 0x0f
	movwf	pos_error

get_drive_data
	movlw	0x04						; Multiply current position by 4 to get correct index in table
	movwf	tempx16
	movf	position,w
	movwf	tempy16
	call	mul16
	movf	result16_H,w				; Set index 2 most significant bits
	movwf	pclath
	bcf 	pclath,4					; set page 1
	bsf 	pclath,3
	movlw	0x01
	addwf	result16_L,f				; increment index
	btfsc	status,c
	incf	result16_H,f
	movf	result16_L,w				; Set index 8 least siginificant bits

	call	drive_table					; get coil drive data
	movwf	drive_byte1					; save byte 1

	movlw	0x01
	addwf	result16_L,f				; increment index
	btfsc	status,c
	incf	result16_H,f
	movf	result16_H,w				; Set index 2 most significant bits
	movwf	pclath
	bcf 	pclath,4					; set page 1
	bsf 	pclath,3
	movf	result16_L,w				; Set index 8 least siginificant bits	
	call	drive_table					; get coil drive data	
	movwf	drive_byte2					; save byte 2

	movlw	0x01
	addwf	result16_L,f				; increment index
	btfsc	status,c
	incf	result16_H,f
	movf	result16_H,w				; Set index 2 most significant bits
	movwf	pclath
	bcf 	pclath,4					; set page 1
	bsf 	pclath,3
	movf	result16_L,w				; Set index 8 least siginificant bits
	call	drive_table					; get coil drive data	
	movwf	drive_byte3					; save byte 3

	movlw	0x01
	addwf	result16_L,f				; increment index
	btfsc	status,c
	incf	result16_H,f
	movf	result16_H,w				; Set index 2 most significant bits
	movwf	pclath
	bcf 	pclath,4					; set page 1
	bsf 	pclath,3
	movf	result16_L,w				; Set index 8 least siginificant bits
	call	drive_table					; get coil drive data	
	movwf	drive_byte4					; save byte 4

	clrf	pclath						; Clear page index

apply_control

_check_coil_1
	btfss	drive_byte1,7				; see if msb = 0
	goto	_check_coil_1_msb_0			; yes
	btfss	drive_byte1,6				; no, see if type 10
	goto	_check_coil_1_10			; yes,
_check_coil_1_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_1_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM01_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_1_end			; continue
_check_coil_1_type_11_zerror
	movwf	PWM01_SP					; yes, drive coil positive
	goto	_check_coil_1_end			; continue
_check_coil_1_msb_0
	btfss	drive_byte1,6				; see if type 01
	goto	_check_coil_1_end			; no, get here if error
_check_coil_1_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_1_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_1_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM01_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_1_end
_check_coil_1_01_zerror
	movwf	PWM01_SP					; yes, drive coil to zero
	goto	_check_coil_1_end
_check_coil_1_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM01_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_1_end		
_check_coil_1_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_1_10_zerror ;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_1_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM01_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_1_end
_check_coil_1_10_zerror
	movwf	PWM01_SP					; yes, drive coil to zero
	goto	_check_coil_1_end
_check_coil_1_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM01_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_1_end
_check_coil_1_end

_check_coil_2
	btfss	drive_byte1,5				; see if msb = 0
	goto	_check_coil_2_msb_0			; yes
	btfss	drive_byte1,4				; no, see if type 10
	goto	_check_coil_2_10			; yes,
_check_coil_2_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_2_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM02_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_2_end			; continue
_check_coil_2_type_11_zerror
	movwf	PWM02_SP					; yes, drive coil positive
	goto	_check_coil_2_end			; continue
_check_coil_2_msb_0
	btfss	drive_byte1,4				; see if type 01
	goto	_check_coil_2_end			; no, get here if error
_check_coil_2_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_2_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_2_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM02_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_2_end
_check_coil_2_01_zerror
	movwf	PWM02_SP					; yes, drive coil to zero
	goto	_check_coil_2_end
_check_coil_2_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM02_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_2_end		
_check_coil_2_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_2_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_2_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM02_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_2_end
_check_coil_2_10_zerror
	movwf	PWM02_SP					; yes, drive coil to zero
	goto	_check_coil_2_end
_check_coil_2_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM02_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_2_end
_check_coil_2_end

_check_coil_3
	btfss	drive_byte1,3				; see if msb = 0
	goto	_check_coil_3_msb_0			; yes
	btfss	drive_byte1,2				; no, see if type 10
	goto	_check_coil_3_10			; yes,
_check_coil_3_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_3_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM03_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_3_end			; continue
_check_coil_3_type_11_zerror
	movwf	PWM03_SP					; yes, drive coil positive
	goto	_check_coil_3_end			; continue
_check_coil_3_msb_0
	btfss	drive_byte1,2				; see if type 01
	goto	_check_coil_3_end			; no, get here if error
_check_coil_3_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_3_01_zerror ;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_3_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM03_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_3_end
_check_coil_3_01_zerror
	movwf	PWM03_SP					; yes, drive coil to zero
	goto	_check_coil_3_end
_check_coil_3_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM03_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_3_end		
_check_coil_3_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_3_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_3_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM03_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_3_end
_check_coil_3_10_zerror
	movwf	PWM03_SP					; yes, drive coil to zero
	goto	_check_coil_3_end
_check_coil_3_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM03_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_3_end
_check_coil_3_end


_check_coil_4
	btfss	drive_byte1,1				; see if msb = 0
	goto	_check_coil_4_msb_0			; yes
	btfss	drive_byte1,0				; no, see if type 10
	goto	_check_coil_4_10			; yes,
_check_coil_4_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_4_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM04_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_4_end			; continue
_check_coil_4_type_11_zerror
	movwf	PWM04_SP					; yes, drive coil positive
	goto	_check_coil_4_end			; continue
_check_coil_4_msb_0
	btfss	drive_byte1,0				; see if type 01
	goto	_check_coil_4_end			; no, get here if error
_check_coil_4_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_4_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_4_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM04_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_4_end
_check_coil_4_01_zerror
	movwf	PWM04_SP					; yes, drive coil to zero
	goto	_check_coil_4_end
_check_coil_4_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM04_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_4_end		
_check_coil_4_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_4_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_4_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM04_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_4_end
_check_coil_4_10_zerror
	movwf	PWM04_SP					; yes, drive coil to zero
	goto	_check_coil_4_end
_check_coil_4_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM04_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_4_end
_check_coil_4_end

_check_coil_5
	btfss	drive_byte2,7				; see if msb = 0
	goto	_check_coil_5_msb_0			; yes
	btfss	drive_byte2,6				; no, see if type 10
	goto	_check_coil_5_10			; yes,
_check_coil_5_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_5_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM05_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_5_end			; continue
_check_coil_5_type_11_zerror
	movwf	PWM05_SP					; yes, drive coil positive
	goto	_check_coil_5_end			; continue
_check_coil_5_msb_0
	btfss	drive_byte2,6				; see if type 01
	goto	_check_coil_5_end			; no, get here if error
_check_coil_5_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_5_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_5_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM05_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_5_end
_check_coil_5_01_zerror
	movwf	PWM05_SP					; yes, drive coil to zero
	goto	_check_coil_5_end
_check_coil_5_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM05_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_5_end		
_check_coil_5_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_5_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_5_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM05_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_5_end
_check_coil_5_10_zerror
	movwf	PWM05_SP					; yes, drive coil to zero
	goto	_check_coil_5_end
_check_coil_5_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM05_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_5_end
_check_coil_5_end

_check_coil_6
	btfss	drive_byte2,5				; see if msb = 0
	goto	_check_coil_6_msb_0			; yes
	btfss	drive_byte2,4				; no, see if type 10
	goto	_check_coil_6_10			; yes,
_check_coil_6_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_6_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM06_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_6_end			; continue
_check_coil_6_type_11_zerror
	movwf	PWM06_SP					; yes, drive coil positive
	goto	_check_coil_6_end			; continue
_check_coil_6_msb_0
	btfss	drive_byte2,4				; see if type 01
	goto	_check_coil_6_end			; no, get here if error
_check_coil_6_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_6_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_6_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM06_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_6_end
_check_coil_6_01_zerror
	movwf	PWM06_SP					; yes, drive coil to zero
	goto	_check_coil_6_end
_check_coil_6_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM06_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_6_end		
_check_coil_6_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_6_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_6_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM06_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_6_end
_check_coil_6_10_zerror
	movwf	PWM06_SP					; yes, drive coil to zero
	goto	_check_coil_6_end
_check_coil_6_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM06_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_6_end
_check_coil_6_end

_check_coil_7
	btfss	drive_byte2,3				; see if msb = 0
	goto	_check_coil_7_msb_0			; yes
	btfss	drive_byte2,2				; no, see if type 10
	goto	_check_coil_7_10			; yes,
_check_coil_7_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_7_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM07_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_7_end			; continue
_check_coil_7_type_11_zerror
	movwf	PWM07_SP					; yes, drive coil positive
	goto	_check_coil_7_end			; continue
_check_coil_7_msb_0
	btfss	drive_byte2,2				; see if type 01
	goto	_check_coil_7_end			; no, get here if error
_check_coil_7_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_7_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_7_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM07_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_7_end
_check_coil_7_01_zerror
	movwf	PWM07_SP					; yes, drive coil to zero
	goto	_check_coil_7_end
_check_coil_7_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM07_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_7_end		
_check_coil_7_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_7_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_7_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM07_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_7_end
_check_coil_7_10_zerror
	movwf	PWM07_SP					; yes, drive coil to zero
	goto	_check_coil_7_end
_check_coil_7_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM07_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_7_end
_check_coil_7_end


_check_coil_8
	btfss	drive_byte2,1				; see if msb = 0
	goto	_check_coil_8_msb_0			; yes
	btfss	drive_byte2,0				; no, see if type 10
	goto	_check_coil_8_10			; yes,
_check_coil_8_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_8_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM08_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_8_end			; continue
_check_coil_8_type_11_zerror
	movwf	PWM08_SP					; yes, drive coil positive
	goto	_check_coil_8_end			; continue
_check_coil_8_msb_0
	btfss	drive_byte2,0				; see if type 01
	goto	_check_coil_8_end			; no, get here if error
_check_coil_8_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_8_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_8_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM08_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_8_end
_check_coil_8_01_zerror
	movwf	PWM08_SP					; yes, drive coil to zero
	goto	_check_coil_8_end
_check_coil_8_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM08_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_8_end		
_check_coil_8_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_8_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_8_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM08_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_8_end
_check_coil_8_10_zerror
	movwf	PWM08_SP					; yes, drive coil to zero
	goto	_check_coil_8_end
_check_coil_8_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM08_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_8_end
_check_coil_8_end


_check_coil_9
	btfss	drive_byte3,7				; see if msb = 0
	goto	_check_coil_9_msb_0			; yes
	btfss	drive_byte3,6				; no, see if type 10
	goto	_check_coil_9_10			; yes,
_check_coil_9_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_9_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM09_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_9_end			; continue
_check_coil_9_type_11_zerror
	movwf	PWM09_SP					; yes, drive coil positive
	goto	_check_coil_9_end			; continue
_check_coil_9_msb_0
	btfss	drive_byte3,6				; see if type 01
	goto	_check_coil_9_end			; no, get here if error
_check_coil_9_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_9_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_9_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM09_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_9_end
_check_coil_9_01_zerror
	movwf	PWM09_SP					; yes, drive coil to zero
	goto	_check_coil_9_end
_check_coil_9_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM09_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_9_end		
_check_coil_9_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_9_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_9_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM09_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_9_end
_check_coil_9_10_zerror
	movwf	PWM09_SP					; yes, drive coil to zero
	goto	_check_coil_9_end
_check_coil_9_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM09_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_9_end
_check_coil_9_end

_check_coil_10
	btfss	drive_byte3,5				; see if msb = 0
	goto	_check_coil_10_msb_0		; yes
	btfss	drive_byte3,4				; no, see if type 10
	goto	_check_coil_10_10			; yes,
_check_coil_10_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_10_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM10_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_10_end			; continue
_check_coil_10_type_11_zerror
	movwf	PWM10_SP					; yes, drive coil positive
	goto	_check_coil_10_end			; continue
_check_coil_10_msb_0
	btfss	drive_byte3,4				; see if type 01
	goto	_check_coil_10_end			; no, get here if error
_check_coil_10_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_10_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_10_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM10_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_10_end
_check_coil_10_01_zerror
	movwf	PWM10_SP					; yes, drive coil to zero
	goto	_check_coil_10_end
_check_coil_10_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM10_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_10_end		
_check_coil_10_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_10_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_10_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM10_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_10_end
_check_coil_10_10_zerror
	movwf	PWM10_SP					; yes, drive coil to zero
	goto	_check_coil_10_end
_check_coil_10_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM10_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_10_end
_check_coil_10_end

_check_coil_11
	btfss	drive_byte3,3				; see if msb = 0
	goto	_check_coil_11_msb_0		; yes
	btfss	drive_byte3,2				; no, see if type 10
	goto	_check_coil_11_10			; yes,
_check_coil_11_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_11_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM11_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_11_end			; continue
_check_coil_11_type_11_zerror
	movwf	PWM11_SP					; yes, drive coil positive
	goto	_check_coil_11_end			; continue
_check_coil_11_msb_0
	btfss	drive_byte3,2				; see if type 01
	goto	_check_coil_11_end			; no, get here if error
_check_coil_11_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_11_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_11_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM11_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_11_end
_check_coil_11_01_zerror
	movwf	PWM11_SP					; yes, drive coil to zero
	goto	_check_coil_11_end
_check_coil_11_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM11_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_11_end		
_check_coil_11_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_11_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_11_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM11_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_11_end
_check_coil_11_10_zerror
	movwf	PWM11_SP					; yes, drive coil to zero
	goto	_check_coil_11_end
_check_coil_11_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM11_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_11_end
_check_coil_11_end


_check_coil_12
	btfss	drive_byte3,1				; see if msb = 0
	goto	_check_coil_12_msb_0		; yes
	btfss	drive_byte3,0				; no, see if type 10
	goto	_check_coil_12_10			; yes,
_check_coil_12_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_12_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM12_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_12_end			; continue
_check_coil_12_type_11_zerror
	movwf	PWM12_SP					; yes, drive coil positive
	goto	_check_coil_12_end			; continue
_check_coil_12_msb_0
	btfss	drive_byte3,0				; see if type 01
	goto	_check_coil_12_end			; no, get here if error
_check_coil_12_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_12_01_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_12_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM12_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_12_end
_check_coil_12_01_zerror
	movwf	PWM12_SP					; yes, drive coil to zero
	goto	_check_coil_12_end
_check_coil_12_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM12_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_12_end		
_check_coil_12_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_12_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_12_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM12_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_12_end
_check_coil_12_10_zerror
	movwf	PWM12_SP					; yes, drive coil to zero
	goto	_check_coil_12_end
_check_coil_12_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM12_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_12_end
_check_coil_12_end

_check_coil_13
	btfss	drive_byte4,7				; see if msb = 0
	goto	_check_coil_13_msb_0		; yes
	btfss	drive_byte4,6				; no, see if type 10
	goto	_check_coil_13_10			; yes,
_check_coil_13_type_11					; get here if type 11
	movlw	0x1f
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_13_type_11_zerror; yes
	movf	pos_error,w					; no, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM13_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_13_end			; continue
_check_coil_13_type_11_zerror
	movwf	PWM13_SP					; yes, drive coil positive
	goto	_check_coil_13_end			; continue
_check_coil_13_msb_0
	btfss	drive_byte4,6				; see if type 01
	goto	_check_coil_13_end			; no, get here if error
_check_coil_13_01						; yes
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_13_01_zerror ;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_13_01_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM13_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_13_end
_check_coil_13_01_zerror
	movwf	PWM13_SP					; yes, drive coil to zero
	goto	_check_coil_13_end
_check_coil_13_01_dn_rq
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM13_SP					; move result to coil PWM setpoint (16-31)
	goto	_check_coil_13_end		
_check_coil_13_10
	movlw	0x0f						; move 15 to w in case error = 0
	movf	pos_error,f					; see if error = 0
	btfsc	status,z
	goto	_check_coil_13_10_zerror 	;
	btfss	flags,move_up_req			; no, see if up movement required
	goto	_check_coil_13_10_dn_rq		; no, down movement required
	movf	pos_error,w					; yes, 0x0f + pos_error -> w
	addlw	0x0f
	movwf	PWM13_SP					; move result to coil PWM setpoint (16-31)	
	goto	_check_coil_13_end
_check_coil_13_10_zerror
	movwf	PWM13_SP					; yes, drive coil to zero
	goto	_check_coil_13_end
_check_coil_13_10_dn_rq
	movf	pos_error,w					; yes, 0x0f - pos_error -> w
	sublw	0x0f
	movwf	PWM13_SP					; move result to coil PWM setpoint (0-15)
	goto	_check_coil_13_end
_check_coil_13_end


end_program
	goto	main
		

;;;; MAIN PROGRAMME
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; TABLE DATA

;; Table data for the angle-to-position (sine) conversion is located between 0x0400 and 0x07ff	
;; pclath should be cleared as we are in page 0
;; Inputs: AD conversion results in ADRESL and ADRESH
;; Outputs: position in mm in w (0=fully up)
;;
;; The table has been built using the following steps:
;; 1. Find fully up voltage (-89mm). Vup.
;; 2. Find fully down voltage (+89mm). Vdn.
;; 3. Find centre voltage. Vc.
;; 4. Configure wiring so that Vup<Vdn
;; 5. Range = (Vdn-Vup)/90 volts per degree, or [(Vdn-Vup)/90]*(1024/5) counts per degree
;; 6. Zero = (Vup/5)*1024 counts
;; 7. Angle = [(AD count  Zero count) / counts per degree]-45
;; 8. Position x=125.86*sin(angle)+89
;; 9. Build Table. I.e:
;;    AD Count : Position (mm)
;;    AD count : 125.86*sin([(AD count  Zero count) / counts per degree]-45)+89
;;
;; Position is 0mm fully up and 178mm fully down.

	


	ORG	0x0400			
pos_table1
	addwf	pcl,f
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
;	retlw 0xBC





	ORG	0x0500			; Address 
pos_table2
	addwf	pcl,f
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBC
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBB
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xBA
	retlw 0xB9
	retlw 0xB9
	retlw 0xB9
	retlw 0xB9
	retlw 0xB9
	retlw 0xB9
	retlw 0xB9
	retlw 0xB8
	retlw 0xB8
	retlw 0xB8
	retlw 0xB8
	retlw 0xB8
	retlw 0xB8
	retlw 0xB7
	retlw 0xB7
	retlw 0xB7
	retlw 0xB7
	retlw 0xB7
	retlw 0xB6
	retlw 0xB6
	retlw 0xB6
	retlw 0xB6
	retlw 0xB6
	retlw 0xB5
	retlw 0xB5
	retlw 0xB5
	retlw 0xB5
	retlw 0xB4
	retlw 0xB4
	retlw 0xB4
	retlw 0xB4
	retlw 0xB3
	retlw 0xB3
	retlw 0xB3
	retlw 0xB3
	retlw 0xB2
	retlw 0xB2
	retlw 0xB2
	retlw 0xB2
	retlw 0xB1
	retlw 0xB1
	retlw 0xB1
	retlw 0xB0
	retlw 0xB0
	retlw 0xB0
	retlw 0xB0
	retlw 0xAF
	retlw 0xAF
	retlw 0xAF
	retlw 0xAE
	retlw 0xAE
	retlw 0xAE
	retlw 0xAD
	retlw 0xAD
	retlw 0xAD
	retlw 0xAC
	retlw 0xAC
	retlw 0xAC
	retlw 0xAB
	retlw 0xAB
	retlw 0xAB
	retlw 0xAA
	retlw 0xAA
	retlw 0xAA
	retlw 0xA9
	retlw 0xA9
	retlw 0xA9
	retlw 0xA8
	retlw 0xA8
	retlw 0xA8
	retlw 0xA7
	retlw 0xA7
	retlw 0xA6
	retlw 0xA6
	retlw 0xA6
	retlw 0xA5
	retlw 0xA5
	retlw 0xA4
	retlw 0xA4
	retlw 0xA4
	retlw 0xA3
	retlw 0xA3
	retlw 0xA2
	retlw 0xA2
	retlw 0xA2
	retlw 0xA1
	retlw 0xA1
	retlw 0xA0
	retlw 0xA0
	retlw 0x9F
	retlw 0x9F
	retlw 0x9F
	retlw 0x9E
	retlw 0x9E
	retlw 0x9D
	retlw 0x9D
	retlw 0x9C
	retlw 0x9C
	retlw 0x9B
	retlw 0x9B
	retlw 0x9A
	retlw 0x9A
	retlw 0x9A
	retlw 0x99
	retlw 0x99
	retlw 0x98
	retlw 0x98
	retlw 0x97
	retlw 0x97
	retlw 0x96
	retlw 0x96
	retlw 0x95
	retlw 0x95
	retlw 0x94
	retlw 0x94
	retlw 0x93
	retlw 0x93
	retlw 0x92
	retlw 0x92
	retlw 0x91
	retlw 0x91
	retlw 0x90
	retlw 0x90
	retlw 0x8F
	retlw 0x8F
	retlw 0x8E
	retlw 0x8E
;	retlw 0x8D





	ORG	0x0600			 
pos_table3
	addwf	pcl,f
	retlw 0x8C
	retlw 0x8C
	retlw 0x8B
	retlw 0x8B
	retlw 0x8A
	retlw 0x8A
	retlw 0x89
	retlw 0x89
	retlw 0x88
	retlw 0x88
	retlw 0x87
	retlw 0x86
	retlw 0x86
	retlw 0x85
	retlw 0x85
	retlw 0x84
	retlw 0x84
	retlw 0x83
	retlw 0x83
	retlw 0x82
	retlw 0x81
	retlw 0x81
	retlw 0x80
	retlw 0x80
	retlw 0x7F
	retlw 0x7F
	retlw 0x7E
	retlw 0x7D
	retlw 0x7D
	retlw 0x7C
	retlw 0x7C
	retlw 0x7B
	retlw 0x7A
	retlw 0x7A
	retlw 0x79
	retlw 0x79
	retlw 0x78
	retlw 0x77
	retlw 0x77
	retlw 0x76
	retlw 0x76
	retlw 0x75
	retlw 0x74
	retlw 0x74
	retlw 0x73
	retlw 0x73
	retlw 0x72
	retlw 0x71
	retlw 0x71
	retlw 0x70
	retlw 0x70
	retlw 0x6F
	retlw 0x6E
	retlw 0x6E
	retlw 0x6D
	retlw 0x6C
	retlw 0x6C
	retlw 0x6B
	retlw 0x6B
	retlw 0x6A
	retlw 0x69
	retlw 0x69
	retlw 0x68
	retlw 0x67
	retlw 0x67
	retlw 0x66
	retlw 0x66
	retlw 0x65
	retlw 0x64
	retlw 0x64
	retlw 0x63
	retlw 0x62
	retlw 0x62
	retlw 0x61
	retlw 0x61
	retlw 0x60
	retlw 0x5F
	retlw 0x5F
	retlw 0x5E
	retlw 0x5D
	retlw 0x5D
	retlw 0x5C
	retlw 0x5B
	retlw 0x5B
	retlw 0x5A
	retlw 0x59
	retlw 0x59
	retlw 0x58
	retlw 0x58
	retlw 0x57
	retlw 0x56
	retlw 0x56
	retlw 0x55
	retlw 0x54
	retlw 0x54
	retlw 0x53
	retlw 0x52
	retlw 0x52
	retlw 0x51
	retlw 0x50
	retlw 0x50
	retlw 0x4F
	retlw 0x4E
	retlw 0x4E
	retlw 0x4D
	retlw 0x4D
	retlw 0x4C
	retlw 0x4C
	retlw 0x4B
	retlw 0x4B
	retlw 0x4A
	retlw 0x49
	retlw 0x49
	retlw 0x48
	retlw 0x48
	retlw 0x47
	retlw 0x47
	retlw 0x46
	retlw 0x45
	retlw 0x45
	retlw 0x44
	retlw 0x44
	retlw 0x43
	retlw 0x43
	retlw 0x42
	retlw 0x41
	retlw 0x41
	retlw 0x40
	retlw 0x40
	retlw 0x3F
	retlw 0x3F
	retlw 0x3E
	retlw 0x3D
	retlw 0x3D
	retlw 0x3C
	retlw 0x3C
	retlw 0x3B
	retlw 0x3B
	retlw 0x3A
	retlw 0x3A
	retlw 0x39
	retlw 0x38
	retlw 0x38
	retlw 0x37
	retlw 0x37
	retlw 0x36
	retlw 0x36
	retlw 0x35
	retlw 0x35
	retlw 0x34
	retlw 0x33
	retlw 0x33
	retlw 0x32
	retlw 0x32
	retlw 0x31
	retlw 0x31
	retlw 0x30
	retlw 0x30
	retlw 0x2F
	retlw 0x2F
	retlw 0x2E
	retlw 0x2D
	retlw 0x2D
	retlw 0x2C
	retlw 0x2C
	retlw 0x2B
	retlw 0x2B
	retlw 0x2A
	retlw 0x2A
	retlw 0x29
	retlw 0x29
	retlw 0x28
	retlw 0x28
	retlw 0x27
	retlw 0x27
	retlw 0x26
	retlw 0x26
	retlw 0x25
	retlw 0x24
	retlw 0x24
	retlw 0x23
	retlw 0x23
	retlw 0x22
	retlw 0x22
	retlw 0x21
	retlw 0x21
	retlw 0x20
	retlw 0x20
	retlw 0x1F
	retlw 0x1F
	retlw 0x1E
	retlw 0x1E
	retlw 0x1D
	retlw 0x1D
	retlw 0x1C
	retlw 0x1C
	retlw 0x1B
	retlw 0x1B
	retlw 0x1A
	retlw 0x1A
	retlw 0x19
	retlw 0x19
	retlw 0x18
	retlw 0x18
	retlw 0x18
	retlw 0x17
	retlw 0x17
	retlw 0x16
	retlw 0x16
	retlw 0x15
	retlw 0x15
	retlw 0x14
	retlw 0x14
	retlw 0x13
	retlw 0x13
	retlw 0x12
	retlw 0x12
	retlw 0x11
	retlw 0x11
	retlw 0x11
	retlw 0x10
	retlw 0x10
	retlw 0x0F
	retlw 0x0F
	retlw 0x0E
	retlw 0x0E
	retlw 0x0D
	retlw 0x0D
	retlw 0x0D
	retlw 0x0C
	retlw 0x0C
	retlw 0x0B
	retlw 0x0B
	retlw 0x0A
	retlw 0x0A
	retlw 0x0A
	retlw 0x09
	retlw 0x09
	retlw 0x08
	retlw 0x08
	retlw 0x08
	retlw 0x07
	retlw 0x07
	retlw 0x06
	retlw 0x06
	retlw 0x06
	retlw 0x05
	retlw 0x05
	retlw 0x04
	retlw 0x04
	retlw 0x04
	retlw 0x03
	retlw 0x03
	retlw 0x02
	retlw 0x02
;	retlw 0x02





	ORG	0x0700			
pos_table4
	addwf	pcl,f
	retlw 0x01
	retlw 0x01
	retlw 0x01
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00
	retlw 0x00






;; Table data for position vs driving force. This data drives the coils depending on the position.
;; Data consists of 4 bytes for every position. These
;; bytes hold driving data for each of the 13 coil arrangements. The data for each coil is 2 bits 
;; long which represents one of four possible driving scenarios. 
;; 00 = zero - no driving force
;; 01 = to raise position, set PWM on that coil to 0-15, to lower position, set PWM on that coil to 16-31, to hold position set PWM on that coil to zero
;; 10 = to raise position, set PWM on that coil to 15-31, to lower position, set PWM on that coil to 0-15, to hold position set PWM on that coil to zero
;; 11 = to raise position, set PWM on that coil to 0-15, to lower position, set PWM on that coil to 0-15, to hold position set PWM on that coil to 15-31
;;
;; For position XXX mm
;;
;; 1st Byte 
;; Bit 76     54     32     10
;;     coil01 coil02 coil03 coil04
;;
;; 2nd Byte  
;; Bit 76     54     32     10
;;     coil05 coil06 coil07 coil08
;;
;; 3rd Byte  
;; Bit 76     54     32     10
;;     coil09 coil10 coil11 coil12
;;
;; 4th Byte  
;; Bit 76     54     32     10
;;     coil13 N/A     N/A     N/A
;;


	ORG	0x0800			; Address 2048
drive_table
	movwf	pcl ;,f
						; Position 0mm
	retlw B'10101010'	
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01111010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10010110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10110110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11100110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100111'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100101'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101101'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01111001'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11011001'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'11101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011011'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011110'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10110110'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11100110'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01111010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'11011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100111'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101101'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01111001'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11011001'
	retlw B'10011010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10010110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'11100110'
	retlw B'01100111'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01100101'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01101101'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01111001'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'11011001'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01100111'
	retlw B'10011001'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01101101'
	retlw B'10011001'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'01111001'
	retlw B'10011001'
	retlw B'11101010'
	retlw B'10000000'
	retlw B'11011001'
	retlw B'10011001'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10011011'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'01101010'
	retlw B'10000000'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'01111010'
	retlw B'10000000'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'01011010'
	retlw B'10000000'
	retlw B'11100110'
	retlw B'01100110'
	retlw B'11011010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01100111'
	retlw B'10011010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01101101'
	retlw B'10011010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'01111001'
	retlw B'10011010'
	retlw B'10000000'
	retlw B'01100110'
	retlw B'11011001'
	retlw B'10011010'
	retlw B'10000000'
	retlw B'01100111'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'10000000'
	retlw B'01101101'
	retlw B'10011001'
	retlw B'10010110'
	retlw B'10000000'
	retlw B'01111001'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'10000000'
	retlw B'01011001'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'10000000'
	retlw B'11011001'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'10000000'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'01100111'
	retlw B'10000000'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'01100101'
	retlw B'10000000'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'01101101'
	retlw B'10000000'
	retlw B'10010110'
	retlw B'01100110'
	retlw B'01111001'
	retlw B'10000000'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'11011001'
	retlw B'10000000'
	retlw B'10100110'
	retlw B'01100111'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10100110'
	retlw B'01101101'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10100110'
	retlw B'01111001'
	retlw B'10011001'
	retlw B'11000000'
	retlw B'10100110'
	retlw B'11011001'
	retlw B'10011001'
	retlw B'01000000'
	retlw B'10100111'
	retlw B'10011001'
	retlw B'10011011'
	retlw B'01000000'
	retlw B'10100101'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'01000000'
	retlw B'10101101'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'01000000'
	retlw B'10101001'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'01000000'
	retlw B'10101001'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101001'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101001'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101001'
	retlw B'11100110'
	retlw B'01100110'
	retlw B'11000000'
	retlw B'10101001'
	retlw B'01100110'
	retlw B'01100111'
	retlw B'10000000'
	retlw B'10101011'
	retlw B'01100110'
	retlw B'01101101'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01100110'
	retlw B'01111001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01100110'
	retlw B'11011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01100111'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01101101'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01111001'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'01011001'
	retlw B'10011001'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'11011001'
	retlw B'10011011'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'10011110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'10110110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'11100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10011011'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10011110'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10010110'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10110110'
	retlw B'01100110'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01100111'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01101101'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01111001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'11011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10100111'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10100101'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101101'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10011001'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10011011'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10011110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10110110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'11100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101011'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01100110'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01100111'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01101101'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01111001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'11011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10011001'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10011011'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10011110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10010110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10110110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100110'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100111'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10100101'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101101'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101001'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101011'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'01000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'11000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10101010'
	retlw B'10000000'


	end       	                ; directive 'end of program'


